# -*- coding: utf-8 -*-

"""
mdnotes

This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from enum import Enum
from mdnotes.http.requests_client import RequestsClient


class Environment(Enum):
    """An enum for SDK environments"""
    PRODUCTION = 0


class Server(Enum):
    """An enum for API servers"""
    DEFAULT = 0
    AUTH = 1


class Configuration(object):
    """A class used for configuring the SDK by a user.
    """

    @property
    def http_client(self):
        return self._http_client

    @property
    def timeout(self):
        return self._timeout

    @property
    def max_retries(self):
        return self._max_retries

    @property
    def backoff_factor(self):
        return self._backoff_factor

    @property
    def environment(self):
        return self._environment

    @property
    def o_auth_client_id(self):
        return self._o_auth_client_id

    @property
    def o_auth_redirect_uri(self):
        return self._o_auth_redirect_uri

    def __init__(self, timeout=60, max_retries=3, backoff_factor=0,
                 environment=Environment.PRODUCTION,
                 o_auth_client_id='TODO: Replace',
                 o_auth_redirect_uri='TODO: Replace'):
        # The value to use for connection timeout
        self._timeout = timeout

        # The number of times to retry an endpoint call if it fails
        self._max_retries = max_retries

        # A backoff factor to apply between attempts after the second try.
        # urllib3 will sleep for:
        # `{backoff factor} * (2 ** ({number of total retries} - 1))`
        self._backoff_factor = backoff_factor

        # Current API environment
        self._environment = environment

        # OAuth 2 Client ID
        self._o_auth_client_id = o_auth_client_id

        # OAuth 2 Redirection endpoint or Callback Uri
        self._o_auth_redirect_uri = o_auth_redirect_uri

        # The Http Client to use for making requests.
        self._http_client = self.create_http_client()

    def clone_with(self, timeout=None, max_retries=None, backoff_factor=None,
                   environment=None, o_auth_client_id=None,
                   o_auth_redirect_uri=None):
        timeout = timeout or self.timeout
        max_retries = max_retries or self.max_retries
        backoff_factor = backoff_factor or self.backoff_factor
        environment = environment or self.environment
        o_auth_client_id = o_auth_client_id or self.o_auth_client_id
        o_auth_redirect_uri = o_auth_redirect_uri or self.o_auth_redirect_uri

        return Configuration(timeout=timeout, max_retries=max_retries,
                             backoff_factor=backoff_factor,
                             environment=environment,
                             o_auth_client_id=o_auth_client_id,
                             o_auth_redirect_uri=o_auth_redirect_uri)

    def create_http_client(self):
        return RequestsClient(timeout=self.timeout,
                              max_retries=self.max_retries,
                              backoff_factor=self.backoff_factor)

    # All the environments the SDK can run in
    environments = {
        Environment.PRODUCTION: {
            Server.DEFAULT: 'http://markdown-notes-app.herokuapp.com',
            Server.AUTH: 'http://markdown-notes-app.herokuapp.com/oauth'
        }
    }

    def get_base_uri(self, server=Server.DEFAULT):
        """Generates the appropriate base URI for the environment and the
        server.

        Args:
            server (Configuration.Server): The server enum for which the base
            URI is required.

        Returns:
            String: The base URI.

        """
        return self.environments[self.environment][server]



# -*- coding: utf-8 -*-

"""
mdnotes

This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class User(object):

    """Implementation of the 'User' model.

    TODO: type model description here.

    Attributes:
        id (int): TODO: type description here.
        name (string): TODO: type description here.
        email (string): TODO: type description here.
        created_at (string): TODO: type description here.
        updated_at (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "name": 'name',
        "email": 'email',
        "created_at": 'created_at',
        "updated_at": 'updated_at'
    }

    def __init__(self,
                 id=None,
                 name=None,
                 email=None,
                 created_at=None,
                 updated_at=None,
                 additional_properties={}):
        """Constructor for the User class"""

        # Initialize members of the class
        self.id = id
        self.name = name
        self.email = email
        self.created_at = created_at
        self.updated_at = updated_at

        # Add additional model properties to the instance
        self.additional_properties = additional_properties

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get('id')
        name = dictionary.get('name')
        email = dictionary.get('email')
        created_at = dictionary.get('created_at')
        updated_at = dictionary.get('updated_at')

        # Clean out expected properties from dictionary
        for key in cls._names.values():
            if key in dictionary:
                del dictionary[key]

        # Return an object of this model
        return cls(id,
                   name,
                   email,
                   created_at,
                   updated_at,
                   dictionary)



# -*- coding: utf-8 -*-

"""
mdnotes

This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class Note(object):

    """Implementation of the 'Note' model.

    TODO: type model description here.

    Attributes:
        id (long|int): TODO: type description here.
        title (string): TODO: type description here.
        body (string): TODO: type description here.
        user_id (long|int): TODO: type description here.
        created_at (string): TODO: type description here.
        updated_at (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "title": 'title',
        "body": 'body',
        "user_id": 'user_id',
        "created_at": 'created_at',
        "updated_at": 'updated_at'
    }

    def __init__(self,
                 id=None,
                 title=None,
                 body=None,
                 user_id=None,
                 created_at=None,
                 updated_at=None,
                 additional_properties={}):
        """Constructor for the Note class"""

        # Initialize members of the class
        self.id = id
        self.title = title
        self.body = body
        self.user_id = user_id
        self.created_at = created_at
        self.updated_at = updated_at

        # Add additional model properties to the instance
        self.additional_properties = additional_properties

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get('id')
        title = dictionary.get('title')
        body = dictionary.get('body')
        user_id = dictionary.get('user_id')
        created_at = dictionary.get('created_at')
        updated_at = dictionary.get('updated_at')

        # Clean out expected properties from dictionary
        for key in cls._names.values():
            if key in dictionary:
                del dictionary[key]

        # Return an object of this model
        return cls(id,
                   title,
                   body,
                   user_id,
                   created_at,
                   updated_at,
                   dictionary)



# -*- coding: utf-8 -*-

"""
mdnotes

This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from mdnotes.decorators import lazy_property
from mdnotes.configuration import Configuration
from mdnotes.configuration import Environment
from mdnotes.controllers.service_controller import ServiceController
from mdnotes.controllers.user_controller import UserController


class MdnotesClient(object):

    @lazy_property
    def service(self):
        return ServiceController(self.config)

    @lazy_property
    def user(self):
        return UserController(self.config)

    def __init__(self, timeout=60, max_retries=3, backoff_factor=0,
                 environment=Environment.PRODUCTION,
                 o_auth_client_id='TODO: Replace',
                 o_auth_redirect_uri='TODO: Replace', config=None):
        if config is None:
            self.config = Configuration(timeout=timeout,
                                        max_retries=max_retries,
                                        backoff_factor=backoff_factor,
                                        environment=environment,
                                        o_auth_client_id=o_auth_client_id,
                                        o_auth_redirect_uri=o_auth_redirect_uri)
        else:
            self.config = config



# -*- coding: utf-8 -*-

"""
mdnotes

This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""


class ServiceStatus(object):

    """Implementation of the 'ServiceStatus' model.

    TODO: type model description here.

    Attributes:
        app (string): TODO: type description here.
        moto (string): TODO: type description here.
        notes (int): TODO: type description here.
        users (int): TODO: type description here.
        time (string): TODO: type description here.
        os (string): TODO: type description here.
        php_version (string): TODO: type description here.
        status (string): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "app": 'app',
        "moto": 'moto',
        "notes": 'notes',
        "users": 'users',
        "time": 'time',
        "os": 'os',
        "php_version": 'php_version',
        "status": 'status'
    }

    def __init__(self,
                 app=None,
                 moto=None,
                 notes=None,
                 users=None,
                 time=None,
                 os=None,
                 php_version=None,
                 status=None,
                 additional_properties={}):
        """Constructor for the ServiceStatus class"""

        # Initialize members of the class
        self.app = app
        self.moto = moto
        self.notes = notes
        self.users = users
        self.time = time
        self.os = os
        self.php_version = php_version
        self.status = status

        # Add additional model properties to the instance
        self.additional_properties = additional_properties

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """
        if dictionary is None:
            return None

        # Extract variables from the dictionary
        app = dictionary.get('app')
        moto = dictionary.get('moto')
        notes = dictionary.get('notes')
        users = dictionary.get('users')
        time = dictionary.get('time')
        os = dictionary.get('os')
        php_version = dictionary.get('php_version')
        status = dictionary.get('status')

        # Clean out expected properties from dictionary
        for key in cls._names.values():
            if key in dictionary:
                del dictionary[key]

        # Return an object of this model
        return cls(app,
                   moto,
                   notes,
                   users,
                   time,
                   os,
                   php_version,
                   status,
                   dictionary)



# -*- coding: utf-8 -*-

"""
mdnotes

This file was automatically generated by APIMATIC v2.0 ( https://apimatic.io ).
"""

from mdnotes.api_helper import APIHelper
from mdnotes.configuration import Server
from mdnotes.controllers.base_controller import BaseController
from mdnotes.models.user import User


class UserController(BaseController):

    """A Controller to access Endpoints in the mdnotes API."""

    def __init__(self, config, call_back=None):
        super(UserController, self).__init__(config, call_back)

    def get_user(self):
        """Does a GET request to /api/user.

        TODO: type endpoint description here.

        Returns:
            User: Response from the API.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        # Prepare query URL
        _url_path = '/api/user'
        _query_builder = self.config.get_base_uri()
        _query_builder += _url_path
        _query_url = APIHelper.clean_url(_query_builder)

        # Prepare headers
        _headers = {
            'accept': 'application/json'
        }

        # Prepare and execute request
        _request = self.config.http_client.get(_query_url, headers=_headers)
        _response = self.execute_request(_request)
        self.validate_response(_response)

        decoded = APIHelper.json_deserialize(_response.text, User.from_dictionary)

        return decoded
